---
title: 'starfish: scalable pipelines for image-based transcriptomics'
tags:
  - Python
  - xarray
  - skimage
  - microscopy
  - imaging
  - biology
  - single cell biology
  - spatial transcriptomics
  - MERFISH
  - In Situ Sequencing
  - osmFISH
  - smFISH
  - BaristaSeq
  - dartfish
  - starmap
  - seqFISH
authors:
  - name: Shannon Axelrod
    affiliation: 1
  - name: Matthew Cai
    orcid: 0000-0003-4998-6328
    affiliation: 1
  - name: Ambrose J. Carr
    orcid: 0000-0002-8457-2836
    affiliation: 1
  - name: Jeremy Freeman
    orcid: 0000-0001-7077-7972
    affiliation: 1
  - name: Deep Ganguli
    affiliation: 1
  - name: Justin T. Kiggins
    orcid: 0000-0002-4638-7015
    affiliation: 1
  - name: Brian Long
    orcid: 0000-0002-7793-5969
    affiliation: 2
  - name: Tony Tung
    affiliation: 1
  - name: Kevin A. Yamauchi
    orcid: 0000-0002-7818-1388
    affiliation: 3
affiliations:
 - name: Chan Zuckerberg Initiative
   index: 1
 - name: Allen Institute for Brain Science
   index: 2
 - name: Chan Zuckerberg Biohub
   index: 3
date: 24 April 2020
bibliography: paper.bib
---

# Summary

The exploding field of single cell transcriptomics has begun to enable deep analysis of gene expression and cell types, but spatial context is lost in the preparation of tissue for these assays.
Recent developments in biochemistry, microfluidics, and microscopy have come together to bring about an “alphabet soup” of technologies that enable sampling gene expression in situ, with varying levels of spatial resolution, sensitivity, and genetic depth.
These technologies promise to permit biologists to ask new questions about the spatial relationships between cell type and interactions between gene expression and cell morphology.
However, these assays generate very large microscopy datasets which are challenging to process using general microscopy analysis tools.
Furthermore, many of these assays require specialized analysis to decode gene expression from multiplexed experimental designs.

# Statement of Need

*starfish* is a Python library for processing images generated by microscopy-based spatial transcriptomics assays.
It lets biologists build scalable pipelines that localize and quantify RNA transcripts in image data generated by any hybridization- or sequencing-based *in situ* transcriptomics method, from classic RNA single-molecule FISH to combinatorial barcoded assays.
Image processing of an experiment is divided into fields of view (FOV) that correspond to the data produced by a microscope at a single location on a microscope slide.
*starfish* lets users register and pre-process images in each FOV, localize spots representing tagged RNA molecules in 3D, decode the identity of those molecules according to the experimental design, segment cells, assign the spots to cells, then aggregate spots into a cell x gene expression matrix.
This spatially-annotated gene expression matrix can then be analyzed and visualized in downstream tools for single-cell biology, such as Seurat [@Seurat], Bioconductor [@Bioconductor], Scanpy [@ScanPy], and cellxgene [@cellxgene].

To enable large scale processing of these data, *starfish* leverages a 5-dimensional imaging data model (x, y, z, round, channel) backed by the cloud-friendly `spacetx-format` file format, and [slicedimage](https://github.com/spacetx/slicedimage/), an interface for lazy, distributed loading of `spacetx-format` datasets.  
Furthermore, *starfish* implements comprehensive logging of all data processing steps for provenance tracking, reproducibility, and transparency.
*starfish* is built on top of popular Python tools like xarray [@xarray] and scikit-image [@scikit-image].

There are a number of other tools which support localization and quantification of spots in fluorescent microscopy images, including ImageJ and CellProfiler, however these tools do not support multiplexed decoding of gene targets necessary for many assays. Other tools which are designed more specifically to handle the kinds of assays that starfish supports include [dotdotdot](https://github.com/LieberInstitute/dotdotdot) [@dotdotdot], a MATLAB toolbox designed for RNAscope assays;  [pysmFISH](https://github.com/linnarsson-lab/pysmFISH/), a Python package designed for smFISH assays; and [SMART-Q](https://github.com/shenlab-ucsf/SMART-Q), a fork from an earlier development release of starfish adding support for immunostaining and other features [@SMART-Q].

starfish requires a working knowledge of Python and fluorescent image analysis for a user to create an analysis pipeline.
To help new users get started and support the broader single cell biology community in learning how to work with these data, *starfish* maintains example datasets and reference implementations ported from published assays, including
MERFISH [@MERFISH],
In Situ Sequencing [@ISS],
osmFISH [@osmFISH],
BaristaSeq [@BaristaSeq],
smFISH [@smFISH],
DARTFISH [@DARTFISH],
STARmap [@starmap],
and seqFISH [@seqFISH].
To take advantage of starfish's support for large scale processing, users must have familiarity with cluster or cloud computing.

*starfish* was developed alongside the [SpaceTx project](https://spacetx-starfish.readthedocs.io/en/stable/about/index.html), a CZI-funded effort to compare spatial transcriptomics methods in the context of determining cell types in the brain [@starfish-sfn].
*starfish* is currently in use by multiple research groups, including the [Allen Institute for Brain Science](https://alleninstitute.org), the [Chan Zuckerberg Biohub](https://www.czbiohub.org/), and the [Zhang Lab at UC San Diego](http://jinzhanglab.ucsd.edu/).
These groups support multiple large-scale projects profiling *in situ* gene expression, including the SpaceTx consortium, the [Human Cell Atlas](https://www.humancellatlas.org/), the [BRAIN Initiative Cell Census Network](https://biccn.org/), and the [HuBMAP Consortium](https://hubmapconsortium.org/).


# Acknowledgements

We would like to acknowledge the following contributions, which have been invaluable to the development of starfish.

For direct contributions of code and documentation, we thank
Olga Botvinnik,
Gökçen Eraslan,
Kira Evans,
Marcus Kinsella,
Nicholas Mei,
Josh Moore,
Nicholas Sofroniew,
and Ola Tarkowska.

We would like to thank the members of the SpaceTx consortium for working closely with us to understand their image processing steps, port pipelines into starfish, and contribute example datasets.
We especially would like to thank
Jeff Moffitt and Xiaowei Zhuang for their help with the MERFISH pipeline and example data,
Marco Mignardi and Mats Nilsson for their help with the In Situ Sequencing pipeline and example data,
Simone Codeluppi and Sten Linnarsson for their help with the osmFISH pipeline and example data,
Xioayin Chen and Anthony Zador for their help with the BaristaSeq pipeline and example data,
Richard Que and Kun Zhang for their help with the DARTFISH pipeline and example data,
Xiao Wang, William Allen, and Karl Deisseroth for their help with the STARmap pipeline and example data,
Dan Goodwin and Ed Boyden for their help with the ExFISH pipeline and example data,
Nico Pierson, Sheel Shah, and Long Cai for their help with the seqFISH pipeline and example data,
and Denis Shapiro for their help with the Imaging Mass Cytometry example data.
Brian Long would like to thank the Allen Institute founder, Paul G. Allen, for his vision, encouragement and support.
Finally, we would like to thank the Science and Technology teams and the Single Cell Program at the Chan Zuckerberg Initiative for their support of this work.

# References
